# -*- coding: utf8 -*-
from __future__ import unicode_literals

import logging

from collections import OrderedDict

from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    HprimForeign,
    HprimString,
    Segment,
)

logger = logging.getLogger("driverng.atras")


class AtrasSegmentH(HeaderSegment):
    sender_name = HprimString((5, 2))
    component_number = HprimString((5, 3))
    version = HprimString((5, 4))
    serial_number = HprimString((5, 5))
    receiver_id = HprimString(10)
    processing_id = HprimString(12, default="P")

    queries = HprimForeign("Q", "header")
    orders = HprimForeign("O", "header")
    results = HprimForeign("R", "header")
    end = HprimForeign("L", "header", one2one=True, required=True)


class AtrasSegmentQ(Segment):
    barcode = HprimString((3, 2))
    sorting_mode = HprimString((3, 3))
    cap_color = HprimString((3, 4))
    test_id = HprimString(5)
    stat_entry_status = HprimString(11)
    request_info_status_code = HprimString(13)


class AtrasSegmentO(Segment):
    barcode = HprimString((4, 1))
    sorting_mode = HprimString((4, 2))
    cap_color = HprimString((4, 3))
    free_variable_parameter = HprimString((4, 4), default="N")
    test_id = HprimString((5, 4), necessary=True)
    priority = HprimString(6, default="R")


class AtrasSegmentR(Segment):
    barcode = HprimString((4, 1))
    sorting_mode = HprimString((4, 2))
    cap_color = HprimString((4, 3))
    free_variable_parameter = HprimString((4, 4))
    target_bin_rack = HprimString((4, 5))
    sorting_success = HprimString((4, 6))
    target_id = HprimString((4, 7))
    result_status = HprimString(9)


class AtrasSegmentL(Segment):
    terminaison_code = HprimString(3, default="N")


class Driver(AstmMessageV2):
    SPEC = {"code": "atras", "label": "ATRAS ASTM BECKMAN-COULTER"}
    ENCODING = "UTF-8"

    SEGMENT_CLASS_MAP = {
        "H": AtrasSegmentH,
        "O": AtrasSegmentO,
        "Q": AtrasSegmentQ,
        "R": AtrasSegmentR,
        "L": AtrasSegmentL,
    }

    OPTIONS = [
        AstmMessageV2.Option(
            "atras_sender_name",
            label="Paramétrer le nom du LIS du segment H",
            default="MCA",
        ),
        AstmMessageV2.BooleanOption(
            "atras_version_routage_zone",
            label="Routage du trieur avec envoi de la zone",
            default=False,
        ),
    ]

    def __init__(self, **kwargs):
        self.atras_sender_name = kwargs.pop("atras_sender_name", "MCA")
        self.atras_version_routage_zone = kwargs.pop(
            "atras_version_routage_zone", False
        )
        super(Driver, self).__init__(**kwargs)

    def create(self, **kwargs):
        message = AtrasSegmentH(type_segment="H", **kwargs)
        message.end = AtrasSegmentL(type_segment="L")
        return message

    def encode_worklist(self, worklist, context):
        # on répond uniquement à un query
        message = self.create(
            receiver_id=context["query"].sender_name,
            component_number=context["query"].component_number,
            version=context["query"].version,
            serial_number=context["query"].serial_number,
            sender_name=self.atras_sender_name,
        )

        for seg_q in context["query"].queries:
            seg_o = None
            for patient in worklist.get_astm_worklist(
                unique_together=["priority"], priority={worklist.ELSE: "R"}
            ):
                for sample, priority in patient.samples:
                    for order in sample.orders:
                        if sample.sample_id == seg_q.barcode:
                            if context.get("zone_colisage"):
                                test_id = context.get("zone_colisage")
                            elif order["zone_groupement_test"]:
                                test_id = order["zone_groupement_test"]
                            else:
                                test_id = ""
                            seg_o = message.orders.add(
                                barcode=seg_q.barcode,
                                sorting_mode=seg_q.sorting_mode,
                                cap_color=seg_q.cap_color,
                                test_id=test_id,
                                priority=priority,
                            )
                            break
            if not seg_o:  # inconnu
                seg_o = message.orders.add(
                    barcode=seg_q.barcode,
                    sorting_mode=seg_q.sorting_mode,
                    cap_color=seg_q.cap_color,
                    test_id="99",
                )

        return message

    def decode_message(self, message):
        if message.queries:
            worklist = Worklist(context="query")
            for seg_q in message.queries:
                worklist.get_or_create(type="patient", sample_id=seg_q.barcode)
            return worklist

        worklist = Worklist(context="result")
        for seg_r in message.results:
            bin_rack = seg_r.target_bin_rack.split(".")
            sample = worklist.get_or_create(
                sample_id=seg_r.barcode, type="patient", rack=bin_rack[1]
            )
            results = OrderedDict()
            results["Bin"] = bin_rack[0]
            results["Cap Color"] = seg_r.cap_color
            results["Rack"] = bin_rack[1]
            results["Sorting Success"] = seg_r.sorting_success
            results["Target ID"] = seg_r.target_id
            for key in results:
                sample.results.add(code=key, value=results.get(key))

        return worklist
