# -*- coding: utf8 -*-
from __future__ import unicode_literals

import csv
import datetime
import logging
from six import StringIO

from driverng import MessageV2, Worklist
from driverng.splitted import Multiple, Splitted, String

logger = logging.getLogger("driverng.cfx_biorad")


class CfxBioradBase(Splitted):

    SEPARATOR = ","


class CfxBioradOrder(CfxBioradBase):
    FIELDS = (
        String("field_1"),
        String("field_2"),
        String("field_3"),
        String("field_4"),
        String("field_5"),
        String("field_6"),
        String("field_7"),
        String("field_8"),
        String("field_9"),
        String("field_10"),
        String("field_11"),
        String("field_12"),
        String("field_13"),
        String("field_14"),
        String("field_15"),
        String("field_16"),
        String("field_17"),
        String("field_18"),
        String("field_19"),
        String("field_20"),
        String("field_21"),
        String("field_22"),
        String("field_23"),
        String("field_24"),
        String("field_25"),
        String("field_26"),
        String("field_27"),
        String("field_28"),
        String("field_29"),
        String("field_30"),
    )


class CfxBioradResult(object):
    FIELDS = [
        String("well"),
        String("fluor"),
        String("target"),
        String("content"),
        String("sample"),
        Multiple("results", (String("resu"),), separator=","),
    ]

    def parse(self, line):
        fields = self.FIELDS
        for index, field in enumerate(fields):
            if isinstance(field, Multiple):
                setattr(self, field.name, line[index:])
                break
            else:
                setattr(self, field.name, line[index])


class CfxBioradMultiple(list):
    _MESSAGE_TYPE_MAP = {
        MessageV2.ORDER: CfxBioradOrder,
        MessageV2.RESULT: CfxBioradResult,
    }

    def add(self, type_message=None, **kwargs):
        obj = self._MESSAGE_TYPE_MAP[type_message](**kwargs)
        self.append(obj)
        return obj

    def parse(self, data):
        buf = StringIO(data)
        reader = csv.reader(buf, delimiter=str(","))
        self.end_date = None
        self.file_name = None
        self.operator_user = None
        self.instrument_serial_number = None
        self.notes = None
        for line in reader:
            if not len(line):
                continue
            if line[0] == "Run Ended":
                try:
                    self.end_date = datetime.datetime.strptime(
                        line[1].strip()[:19], "%m/%d/%Y %H:%M:%S"
                    )
                except ValueError as e:
                    logger.info("La date des résultat n'est pas au bon format %s", e)
            if line[0] == "File Name":
                self.file_name = line[1].strip()
            if line[0] == "Created By User":
                self.operator_user = line[1].strip()
            if line[0] == "Base Serial Number":
                self.instrument_serial_number = line[1].strip()
            if line[0] == "Notes":
                self.notes = line[1].strip()
            if len(line) >= 6 and "File Name" in data:
                type_message = MessageV2.RESULT
            else:
                type_message = MessageV2.ORDER
            obj = self.add(type_message=type_message)
            obj.parse(line)

    def to_string(self):
        return "\n".join([e.to_string() for e in self])


class Driver(MessageV2):
    SPEC = {"code": "cfx_biorad", "label": "Cfx Bio-Rad CSV"}

    OPTIONS = [
        MessageV2.Option("analyseur_traites", label="Liste des analyseurs traités"),
        MessageV2.BooleanOption(
            "extra_results", label="Récupérer plusieurs résultats", default=False
        ),
    ]

    MESSAGE_TYPE_MAP = {
        MessageV2.ORDER: CfxBioradOrder,
        MessageV2.RESULT: CfxBioradResult,
    }

    def __init__(self, **kwargs):
        analyseur_traites = kwargs.pop("analyseur_traites", None)
        self.analyseur_traites = (
            analyseur_traites.split(",") if analyseur_traites else None
        )
        self.extra_results = kwargs.pop("extra_results", False)
        super(Driver, self).__init__(**kwargs)

    def create(self):
        return CfxBioradMultiple()

    def to_string(self, message):
        return message.to_string()

    def parse(self, data):
        obj = CfxBioradMultiple()
        obj.parse(data)
        return obj

    def encode_worklist(self, worklist, context):
        msg = self.create()
        samples = worklist.get_plan_de_plaque_worklist()
        if len(samples) > 1:
            raise RuntimeError("Multi-sample not implemented")
        elif len(samples) == 1:
            sample = samples[0]
            plate_size = sample.geometry.get("plate_size") if sample.geometry else ""
            run_protocol = (
                sample.geometry.get("run_protocol") if sample.geometry else ""
            )
            headers = (
                {"field_1": "Plate Header"},
                {"field_1": "Filed", "field_2": "Data", "field_4": "Instruction"},
                {
                    "field_1": "Version",
                    "field_2": "1",
                    "field_4": "Do not modify this field.",
                },
                {
                    "field_1": "Plate Size",
                    "field_2": plate_size,
                    "field_4": "Do not modify this field.",
                },
                {
                    "field_1": "Plate Type",
                    "field_2": "BR Clear",
                    "field_4": "Allowed values (BR White,BR Clear)",
                },
                {
                    "field_1": "Scan Mode",
                    "field_2": "All Channels",
                    "field_4": '"Allowed values (""SYBR/FAM Only"""',
                    "field_5": '" ""All Channels"""',
                    "field_6": '" ""FRET"")"',
                },
                {"field_1": "Units", "field_2": "copy number"},
                {"field_1": "Run ID"},
                {"field_1": "Run Notes"},
                {"field_1": "Run Protocol", "field_2": "{}.prcl".format(run_protocol)},
                {"field_1": "Data File"},
                {"field_1": "TBD"},
                {"field_1": "Plate Data"},
                {
                    "field_1": "Well",
                    "field_2": "Ch1 Dye",
                    "field_3": "Ch2 Dye",
                    "field_4": "Ch3 Dye",
                    "field_5": "Ch4 Dye",
                    "field_6": "Ch5 Dye",
                    "field_7": "FRET",
                    "field_8": "Sample Type",
                    "field_9": "Sample Name",
                    "field_10": "Ch1 Target Name",
                    "field_11": "Ch2 Target Name",
                    "field_12": "Ch3 Target Name",
                    "field_13": "Ch4 Target Name",
                    "field_14": "Ch5 Target Name",
                    "field_15": "FRET Target Name",
                    "field_16": "Biological Set Name",
                    "field_17": "Replicate",
                    "field_18": "Ch1 Quantity",
                    "field_19": "Ch2 Quantity",
                    "field_20": "Ch3 Quantity",
                    "field_21": "Ch4 Quantity",
                    "field_22": "Ch5 Quantity",
                    "field_23": "FRET Quantity",
                    "field_24": "Well Note",
                    "field_25": "Ch1 Well Color",
                    "field_26": "Ch2 Well Color",
                    "field_27": "Ch3 Well Color",
                    "field_28": "Ch4 Well Color",
                    "field_29": "Ch5 Well Color",
                    "field_30": "FRET Well Color",
                },
            )
            for header in headers:
                msg.add(
                    type_message=MessageV2.ORDER,
                    field_1=header.get("field_1"),
                    field_2=header.get("field_2"),
                    field_3=header.get("field_3"),
                    field_4=header.get("field_4"),
                    field_5=header.get("field_5"),
                    field_6=header.get("field_6"),
                    field_7=header.get("field_7"),
                    field_8=header.get("field_8"),
                    field_9=header.get("field_9"),
                    field_10=header.get("field_10"),
                    field_11=header.get("field_11"),
                    field_12=header.get("field_12"),
                    field_13=header.get("field_13"),
                    field_14=header.get("field_14"),
                    field_15=header.get("field_15"),
                    field_16=header.get("field_16"),
                    field_17=header.get("field_17"),
                    field_18=header.get("field_18"),
                    field_19=header.get("field_19"),
                    field_20=header.get("field_20"),
                    field_21=header.get("field_21"),
                    field_22=header.get("field_22"),
                    field_23=header.get("field_23"),
                    field_24=header.get("field_24"),
                    field_25=header.get("field_25"),
                    field_26=header.get("field_26"),
                    field_27=header.get("field_27"),
                    field_28=header.get("field_28"),
                    field_29=header.get("field_29"),
                    field_30=header.get("field_30"),
                )
            replicate = 0
            targets = {}
            for order in sample.orders:
                codes = order.code.split(";")
                for i in range(len(codes)):
                    targets["ch{}_target".format(i + 1)] = codes[i]
            for well in sample.wells:
                if well.well_type == 0:
                    replicate += 1
                else:
                    replicate = 1
                msg.add(
                    type_message=MessageV2.ORDER,
                    field_1="{}{}".format(well.ordinate, well.abscissa),
                    field_2="FAM",
                    field_3="HEX",
                    field_4="Texas Red",
                    field_5="Cy5",
                    field_8=well.well_type,
                    field_9=well.barcode,
                    field_10=targets.get("ch1_target"),
                    field_11=targets.get("ch2_target"),
                    field_12=targets.get("ch3_target"),
                    field_13=targets.get("ch4_target"),
                    field_14=targets.get("ch5_target"),
                    field_17=replicate,
                )
        return msg

    def decode_message(self, message):
        worklist = Worklist(context="result")
        RESULTS_CODES = ()
        for m in message:
            if not m.target and not m.sample:
                continue
            elif m.sample == "Sample" and m.well == "Well":
                RESULTS_CODES = m.results
                continue
            if (
                self.analyseur_traites is not None
                and message.instrument_serial_number not in self.analyseur_traites
            ):
                logger.error(
                    "L'instrument %s n'est pas traité par la connexion"
                    % message.instrument_serial_number
                )
                continue
            if RESULTS_CODES:
                sample = worklist.get_or_create(
                    sample_id=m.sample, rack="CFX-{}".format(m.well), type="patient"
                )
                if message.file_name:
                    sample.results.get_or_create(
                        code="File Name", value=message.file_name[:-5]
                    )
                if message.notes:
                    sample.results.get_or_create(code="Notes", value=message.notes)
                if message.operator_user:
                    sample.results.get_or_create(
                        code="Created By User", value=message.operator_user
                    )
                if m.well:
                    sample.results.get_or_create(code="RACK", value=m.well)
                for index, result in enumerate(m.results):
                    if RESULTS_CODES[index] and result:
                        # Conserver la compatibilité sur l'ancien format avec
                        # présence d'uniquement un seul résultat dans la colonne Cq
                        if not self.extra_results:
                            sample.results.add(
                                code=m.target,
                                value=result,
                                instrument=message.instrument_serial_number,
                                date=message.end_date,
                                operator=message.operator_user,
                                comments=[message.notes],
                            )
                            sample.results.add(
                                code="{}{}".format(m.fluor, m.target),
                                value=result,
                                instrument=message.instrument_serial_number,
                                date=message.end_date,
                                operator=message.operator_user,
                                comments=[message.notes],
                            )
                        else:
                            sample.results.add(
                                code="{}{}{}".format(
                                    m.fluor, m.target, RESULTS_CODES[index].strip()
                                ),
                                value=result,
                                instrument=message.instrument_serial_number,
                                date=message.end_date,
                                operator=message.operator_user,
                                comments=[message.notes],
                            )
        return worklist
